// MIT License
//
// Copyright (c) 2024 vinchent <vinchent@vinchent.xyz>
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

package repo

import (
	"context"

	"git.vinchent.xyz/vinchent/howmuch/internal/howmuch/model"
)

type EventRepository interface {
	Create(ctx context.Context, evEntity *model.EventEntity) (*model.EventEntity, error)

	// UpdateEventByID updates the event related information (name, descriptions)
	UpdateEventByID(ctx context.Context, event *model.EventUpdateEntity) error

	GetByID(ctx context.Context, eventID int) (*model.EventRetrieved, error)

	// related to events of a user
	ListEventsByUserID(ctx context.Context, userID int) ([]*model.EventListRetrieved, error)
}

type ExpenseRepository interface {
	Create()
	Update()
	Delete() // Delete also the related transactions

	ListExpensesByUserID()
	GetByID()
}

type ParticipationRepository interface {
	Create()
	Delete()
	CheckParticipation(ctx context.Context, userID, eventID int) error
}

type TransactionRepository interface {
	Create()
	// Delete() might be handled in the Expense
	// Transaction is a joined entity, we don't provide diret read operation
}
